/*  This file is part of LingoTeach, the Language Teaching program 
 *  Copyright (C) 2001-2003 The LingoTeach Team
 *
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 *  GNU General Public License for more details. 
 * 
 *  You should have received a copy of the GNU General Public License 
 *  along with this program; if not, write to the Free Software 
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 */

#include <stdlib.h>
#include <lingoteach.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /*HAVE_CONFIG_H */

#include "learningpref.h"
#include "util.h"
#include "onebyone.h"
#include "lingoteach-i18n.h"

/* external global variables */
extern lingLesson *lessons;
extern struct lpreferences genLprefs;
extern GtkTooltips *tooltips;


static GtkWidget *box_langs;
static GtkWidget *box_trans;
static GtkWidget *box_play;
static GtkWidget *lbl_langword;
static GtkWidget *lbl_word;

/* the current meaning */
lingMeaning *obocurrent[30];

GtkWidget*
create_obo_box (void)
{
  GtkWidget *tbl_main;
  GtkWidget *sep_obo1;

  tbl_main = gtk_table_new (4, 3, FALSE);
  gtk_widget_show (tbl_main);
  gtk_table_set_row_spacings (GTK_TABLE (tbl_main), 2);
  gtk_table_set_col_spacings (GTK_TABLE (tbl_main), 2);
  
  sep_obo1 = gtk_hseparator_new ();
  gtk_widget_show (sep_obo1);
  gtk_table_attach (GTK_TABLE (tbl_main), sep_obo1, 0, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 1, 4);

  lbl_langword = gtk_label_new ("");
  gtk_widget_show (lbl_langword);
  gtk_table_attach (GTK_TABLE (tbl_main), lbl_langword, 0, 1, 1, 2,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 4, 4);
  gtk_label_set_justify (GTK_LABEL (lbl_langword), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (lbl_langword), 0.1, 0.5);

  lbl_word = gtk_label_new ("");
  gtk_widget_show (lbl_word);
  gtk_table_attach (GTK_TABLE (tbl_main), lbl_word, 1, 2, 1, 2,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);

  sep_obo1 = gtk_hseparator_new ();
  gtk_widget_show (sep_obo1);
  gtk_table_attach (GTK_TABLE (tbl_main), sep_obo1, 0, 3, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 1, 4);

  box_langs = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (box_langs);
  gtk_table_attach (GTK_TABLE (tbl_main), box_langs, 0, 1, 3, 4,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);

  box_trans = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (box_trans);
  gtk_table_attach (GTK_TABLE (tbl_main), box_trans, 1, 2, 3, 4,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  
  box_play = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (box_play);
  gtk_table_attach (GTK_TABLE (tbl_main), box_play, 2, 3, 3, 4,
		    (GtkAttachOptions) (GTK_FILL),
		    (GtkAttachOptions) (0), 2, 2);

  on_btn_obo_next_clicked ();
  return tbl_main;
}

void 
on_btn_obo_next_clicked (void)
{
  GtkWidget  *lbl_lang;
  GtkWidget  *lbl_trans;
  GtkWidget  *btn_play;
  GtkWidget  *lbl_arrow;
  GList      *children;
  gint        i = 0;
  gint        id;
  lingLesson *less;

  PangoFontDescription *font = pango_font_description_new ();
  pango_font_description_set_style (font, PANGO_STYLE_ITALIC);
  pango_font_description_set_weight (font, PANGO_WEIGHT_BOLD);

  if (util_check_settings () == FALSE)
    return;

  less = util_return_lesson (lessons, genLprefs.method);

  while (obocurrent[i] != NULL)
    {
      
#ifdef DEBUG
      printf ("Debug: freeing lingMeaning element %i\n", i);
#endif
      
      ling_meaning_free_meaning (obocurrent[i]);
      i++;
    }
  i = 0;

  /* clear all items of both boxes */
  if ((children = gtk_container_get_children (GTK_CONTAINER (box_langs)))
      != NULL)
    {
      while (children->next != NULL)
	{
	  gtk_object_destroy (GTK_OBJECT (children->data));
	  children = children->next;
	}
      gtk_object_destroy (GTK_OBJECT (children->data));
      g_list_free (children);
      
      children = gtk_container_get_children (GTK_CONTAINER (box_trans));
      while (children->next != NULL)
	{
	  gtk_object_destroy (GTK_OBJECT (children->data));
	  children = children->next;
	}
      gtk_object_destroy (GTK_OBJECT (children->data));
      g_list_free (children);
      
      children = gtk_container_get_children (GTK_CONTAINER (box_play));
      while (children->next != NULL)
	{
	  gtk_object_destroy (GTK_OBJECT (children->data));
	  children = children->next;
	}
      gtk_object_destroy (GTK_OBJECT (children->data));
      g_list_free (children);
    }

  /* new labels */
  obocurrent[0] = 
    ling_meaning_get_another_meaning (less, genLprefs.method, 
				      genLprefs.mlanguage);
  id = obocurrent[0]->id;

  gtk_label_set_text (GTK_LABEL (lbl_langword), obocurrent[0]->language);
  gtk_label_set_text (GTK_LABEL (lbl_word), obocurrent[0]->translation);

  gtk_widget_modify_font (lbl_langword, font);
  while (genLprefs.languages[i] != NULL)
    {
      /* i + 1 needed, because element 0 is already filled */
      obocurrent[i + 1] = ling_meaning_get_by_word_id (less, id, 
    						       genLprefs.languages[i]);
      
      lbl_lang = gtk_label_new (genLprefs.languages[i]);
      gtk_widget_show (lbl_lang);
      gtk_box_pack_start (GTK_BOX (box_langs), lbl_lang, TRUE, TRUE, 0);
      gtk_label_set_justify (GTK_LABEL (lbl_lang), GTK_JUSTIFY_RIGHT);      
      gtk_misc_set_alignment (GTK_MISC (lbl_lang), 0.1, 0.5);
      gtk_widget_modify_font (lbl_lang, font);

      lbl_trans = gtk_label_new (obocurrent[i + 1]->translation);
      gtk_widget_show (lbl_trans);
      gtk_box_pack_start (GTK_BOX (box_trans), lbl_trans, TRUE, TRUE, 0);

      btn_play = gtk_button_new ();
      gtk_widget_show (btn_play);
      gtk_box_pack_start (GTK_BOX (box_play), btn_play, FALSE, FALSE, 0);
      g_signal_connect_swapped (G_OBJECT (btn_play), "clicked", 
				G_CALLBACK (util_on_btn_play_clicked), 
				obocurrent[i + 1]);
      gtk_tooltips_set_tip (tooltips, btn_play, _("Play Sound"), NULL);

      lbl_arrow = gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
      gtk_widget_show (lbl_arrow);
      gtk_container_add (GTK_CONTAINER (btn_play), lbl_arrow);

      i++;
    }
  pango_font_description_free (font);  
  return;
}
